

package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.picking.*;

 /**
  * Class MarkerTDC
  * Defines a MarkerTDC
  */


class MarkerTDC extends BranchGroup
{
  public static final double SPHERE_HEIGHT=1.05;
  public static final float HANDLE_SIZE = 0.02f;
  public static final double PIPE_RADIUS = 0.02;
  int nr;
  GraphOutput parent;
  public TDCHandle handle;
  TransformGroup tg = new TransformGroup();
  TransformGroup tg0 = new TransformGroup();
  double cur_pos;

  /**
   * Constructor for default-values
   */
  public MarkerTDC (int nr, GraphOutput parent)
  {
    this.nr = nr;
    this.parent = parent;
    cur_pos = 0.0;
    // add children
    createComponents(0.0);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the position of the marker is
   *  specified.
   */
  public MarkerTDC(int nr, double pos, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    cur_pos = pos;
    // add children
    createComponents(pos);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the number and the initial position
   *  are specified
   */
  public MarkerTDC(int nr, double pos, Color3f col, float transparency, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    cur_pos = pos;
    createComponents(pos, col, transparency);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
  }

  /**
   *  set the position of the MarkerTDC
   */
  public void setPosition(double pos)
  {
    // pos [0;1]; select the appropr. dataset, and the vector
    // then translate the marker
    cur_pos = pos;
    Vector pos_v = new Vector();
    pos_v = parent.getTDCPosition(pos);

    // normalize positions
    for(int i=0; i<pos_v.size(); i++)
    {
      double a = ((Double)pos_v.elementAt(i)).doubleValue();
      pos_v.set(i, new Double(((Range)parent.scaling_v.elementAt(i)).normalize(a)) );
    }

    handle.setPosition(pos_v);

    Transform3D t3d = new Transform3D();
    t3d.setTranslation(new Vector3d(
	  ((Double)pos_v.elementAt(0)).doubleValue(),
      SPHERE_HEIGHT,
	  ((Double)pos_v.elementAt(1)).doubleValue() ));

	tg.setTransform(t3d);
  }

  /**
   *  redraws the MarkerTDC (if trajectory has changed)
   */
  public void redraw()
  {
    setPosition(cur_pos);
  }



  /**
   *  applies a movement of the MarkerTDC
   */
  public void applyMovement(double delta)
  {
    if(cur_pos+delta >= 0.0 && cur_pos+delta<=1.0)
    {
      cur_pos += delta;
      setPosition(cur_pos);
    }
  }

  /**
   *  returns the current position of the cutting-plane
   */
//  public double getPoint()
//  {
//
//    return cur_pos;
//  }

  /**
   *  set the color and the transparency of the MarkerTDC
   */
  public void setColor(Color3f col, float transparency)
  {
    handle.setColor(col, transparency);
  }


  /**
   * returns the color of the cuttingplane
   */
  public Color3f getColor()
  {
    return handle.getColor();
  }

  /**
   * returns the transparency of the plane
   */
  public float getTransparency()
  {
    return handle.getTransparency();
  }

  /**
   * returns a string containing the description of this
   * instance
   */
  public String getDescription()
  {
    return "Nr. "+nr+" at Time "+getTime();
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos with the Color col and the specified transparency
   */
  private void createComponents(double pos, Color3f col, float transparency)
  {
    createComponents(pos);
    setColor(col, transparency);
  }

  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   */
  public double getPosition()
  {
    return cur_pos;
  }

  /**
   * returns the current time position of the highlighting marker
   */
  public double getTime()
  {
    return parent.time.fitIntoRange(cur_pos);
  }

  /**
   * sets the current time position of the highlighting marker
   */
  public void setTime(double time)
  {
    setPosition(parent.time.normalize(time));
  }


  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   * with max. 3 digits after the dot
   */
  public double getPosition3()
  {
    Integer i = new Integer( (new Double((1000.0*getPosition() ))).intValue() );
	return (i.doubleValue() / 1000.0);
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos
   */
  private void createComponents(double pos)
  {

    Appearance app = new Appearance();
    Color3f objColor = new Color3f(1.0f, 0.5f, 0.0f);

    Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
    app.setMaterial(mat);

    handle = new TDCHandle( PIPE_RADIUS, 6, parent.dimcnt);
    setPosition(0.0);

    BranchGroup bg = new BranchGroup();
    addChild(bg);
    bg.addChild(handle);


//     tg.setCapability(Node.ENABLE_PICK_REPORTING);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    bg.addChild(tg);

    TransformGroup htg = new TransformGroup();
    tg.addChild(htg);
//     Transform3D t0 = new Transform3D();
//     htg.setCapability(Node.ENABLE_PICK_REPORTING);
//     htg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
//     htg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
//     t0.setTranslation(new Vector3d(0.0, CIRCLE_RADIUS, 0.0));
//     htg.setTransform(t0);

    BranchGroup bg1 = new BranchGroup();
    bg1.addChild(new Sphere(HANDLE_SIZE/2.0f, app));
    bg1.setCapability(BranchGroup.ENABLE_PICK_REPORTING);
    htg.addChild(bg1);

    PickTranslateBehaviorTDC mt = new PickTranslateBehaviorTDC(bg1,
                                                parent.canvas,
                                                new BoundingSphere(new Point3d(0.0, SPHERE_HEIGHT, 0.0), (double)HANDLE_SIZE/2.0),
                                                PickObject.USE_BOUNDS, nr,
                                                parent, this);

    htg.addChild(mt);
  }



} // End of class MarkerTDC